@@ -1,4 +1,3 @@
-import random
 import shutil
 from collections.abc import Iterator
 from pathlib import Path
@@ -108,26 +107,31 @@ def print_info(dataset: LuxonisDataset) -> None:
             task_table.add_row(", ".join(task_types))
 
     splits = dataset.get_splits()
+    source_names = dataset.get_source_names()
 
     @group()
     def get_sizes_panel() -> Iterator[RenderableType]:
         if splits is not None:
-            total_files = len(dataset)
-            for split, files in splits.items():
-                split_size = len(files)
+            total_groups = len(dataset) / len(source_names)
+            for split, group in splits.items():
+                split_size = len(group)
                 percentage = (
-                    (split_size / total_files * 100) if total_files > 0 else 0
+                    (split_size / total_groups * 100)
+                    if total_groups > 0
+                    else 0
                 )
                 yield f"[magenta b]{split}: [not b cyan]{split_size:,} [dim]({percentage:.1f}%)[/dim]"
         else:
             yield "[red]No splits found"
         yield Rule()
-        yield f"[magenta b]Total: [not b cyan]{len(dataset)}"
+        yield f"[magenta b]Total: [not b cyan]{int(total_groups)}"
 
     @group()
     def get_panels() -> Iterator[RenderableType]:
         yield f"[magenta b]Name: [not b cyan]{dataset.identifier}"
         yield f"[magenta b]Version: [not b cyan]{dataset.version}"
+        yield f"[magenta b]Bucket Storage: [not b cyan]{dataset.bucket_storage.value}"
+        yield f"[magenta b]Team ID: [not b cyan]{dataset.team_id}"
         yield ""
         yield Panel.fit(get_sizes_panel(), title="Split Sizes")
         yield class_table
@@ -178,20 +182,22 @@ def delete(
 
     if not where:
         print(
-            "[yellow]No deletion target specified (local or remote). Nothing to delete.[/yellow]"
+            "[red]No deletion target specified (local or remote). Nothing to delete.[/red]"
         )
-        raise typer.Exit
+        raise typer.Exit(1)
 
     if not Confirm.ask(
         f"Delete dataset '{name}' with specified bucket '{bucket_storage}' from {where} storage?"
     ):
         raise typer.Exit
 
-    dataset = LuxonisDataset(name, bucket_storage=bucket_storage)
-    dataset.delete_dataset(
+    dataset = LuxonisDataset(
+        name,
+        bucket_storage=bucket_storage,
         delete_local=local,
         delete_remote=remote,
     )
+    dataset.delete_dataset(delete_local=local)
 
     print(
         f"Dataset '{name}' deleted from: "
@@ -331,10 +337,7 @@ def inspect(
     bucket_storage: BucketStorage = bucket_option,
 ):
     """Inspects images and annotations in a dataset."""
-
-    if deterministic:
-        np.random.seed(42)
-        random.seed(42)
+    check_exists(name, bucket_storage)
 
     view = view or ["train"]
     dataset = LuxonisDataset(name, bucket_storage=bucket_storage)
@@ -345,22 +348,39 @@ def inspect(
     )
 
     if aug_config is not None:
-        h, w, _ = loader[0][0].shape
+        sample_img = loader[0][0]
+        img = (
+            next(iter(sample_img.values()))
+            if isinstance(sample_img, dict)
+            else sample_img
+        )
+        h, w = img.shape[:2]
+
         loader.augmentations = loader._init_augmentations(
-            "albumentations", aug_config, h, w, not ignore_aspect_ratio
+            augmentation_engine="albumentations",
+            augmentation_config=aug_config,
+            height=h,
+            width=w,
+            keep_aspect_ratio=not ignore_aspect_ratio,
+            seed=42 if deterministic else None,
         )
 
     if len(dataset) == 0:
         raise ValueError(f"Dataset '{name}' is empty.")
 
     classes = dataset.get_classes()
-    for image, labels in loader:
-        image = image.astype(np.uint8)
-        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
+    prev_windows = set()
+
+    for img, labels in loader:
+        if isinstance(img, dict):
+            images_dict = img
+        else:
+            images_dict = {"image": img}
+
+        current_windows = set(images_dict.keys())
+        for stale_window in prev_windows - current_windows:
+            cv2.destroyWindow(stale_window)
 
-        h, w, _ = image.shape
-        new_h, new_w = int(h * size_multiplier), int(w * size_multiplier)
-        image = cv2.resize(image, (new_w, new_h))
         instance_keys = [
             "/boundingbox",
             "/keypoints",
@@ -369,35 +389,54 @@ def inspect(
         matched_instance_keys = [
             k for k in labels if any(k.endswith(ik) for ik in instance_keys)
         ]
-        if per_instance and matched_instance_keys:
-            extra_keys = [k for k in labels if k not in matched_instance_keys]
-            if extra_keys:
-                print(
-                    f"[yellow]Warning: Ignoring non-instance keys in labels: {extra_keys}[/yellow]"
+
+        for source_name, image in images_dict.items():
+            image = image.astype(np.uint8)
+            image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
+            h, w = image.shape[:2]
+            new_h, new_w = int(h * size_multiplier), int(w * size_multiplier)
+            image = cv2.resize(image, (new_w, new_h))
+
+            if per_instance and matched_instance_keys:
+                extra_keys = [
+                    k for k in labels if k not in matched_instance_keys
+                ]
+                if extra_keys:
+                    print(
+                        f"[yellow]Warning: Ignoring non-instance keys in labels: {extra_keys}[/yellow]"
+                    )
+                n_instances = len(labels[matched_instance_keys[0]])
+                for i in range(n_instances):
+                    instance_labels = {
+                        k: np.expand_dims(v[i], axis=0)
+                        for k, v in labels.items()
+                        if k in matched_instance_keys and len(v) > i
+                    }
+                    instance_image = visualize(
+                        image.copy(),
+                        source_name,
+                        instance_labels,
+                        classes,
+                        blend_all=blend_all,
+                    )
+                    cv2.imshow(source_name, instance_image)
+                    if cv2.waitKey() == ord("q"):
+                        break
+            else:
+                if per_instance:
+                    print(
+                        "[yellow]Warning: Per-instance mode is not supported for this dataset. "
+                        f"Showing all labels in one window for '{source_name}'.[/yellow]"
+                    )
+                labeled_image = visualize(
+                    image, source_name, labels, classes, blend_all=blend_all
                 )
-            n_instances = len(labels[matched_instance_keys[0]])
-            for i in range(n_instances):
-                instance_labels = {
-                    k: np.expand_dims(v[i], axis=0)
-                    for k, v in labels.items()
-                    if k in matched_instance_keys and len(v) > i
-                }
-                instance_image = visualize(
-                    image.copy(), instance_labels, classes, blend_all=blend_all
-                )
-                cv2.imshow("image", instance_image)
-                if cv2.waitKey() == ord("q"):
-                    break
-        else:
-            if per_instance:
-                print(
-                    "[yellow]Warning: Per-instance mode is not supported for this dataset. "
-                    "Showing all labels in one window.[/yellow]"
-                )
-            image = visualize(image, labels, classes, blend_all=blend_all)
-            cv2.imshow("image", image)
-            if cv2.waitKey() == ord("q"):
-                break
+                cv2.imshow(source_name, labeled_image)
+
+        prev_windows = current_windows
+
+        if cv2.waitKey() == ord("q"):
+            break
 
 
 @app.command()
@@ -434,19 +473,6 @@ def export(
             help="Delete an existing `save_dir` before exporting.",
         ),
     ] = False,
-    task_name_to_keep: Annotated[
-        str | None,
-        typer.Option(
-            "--task-name",
-            "-tn",
-            help=(
-                "Name of the single task to export. "
-                "Required when the dataset contains multiple tasks; "
-                "ignored if the dataset has exactly one task."
-            ),
-            show_default=False,
-        ),
-    ] = None,
     max_partition_size_gb: Annotated[
         float | None,
         typer.Option(
@@ -470,17 +496,13 @@ def export(
     ] = False,
     bucket_storage: BucketStorage = bucket_option,
 ):
+    """Export a Luxonis dataset to disk in native format, with optional
+    partitioning and zip compression for flexible storage."""
     save_dir = save_dir or dataset_name
     if delete_existing and Path(save_dir).exists():
         shutil.rmtree(save_dir)
     dataset = LuxonisDataset(dataset_name, bucket_storage=bucket_storage)
-    dataset.export(
-        save_dir,
-        dataset_type,
-        task_name_to_keep,
-        max_partition_size_gb,
-        not no_zip,
-    )
+    dataset.export(save_dir, dataset_type, max_partition_size_gb, not no_zip)
 
 
 @app.command()
@@ -674,6 +696,12 @@ def health(
 
     console.print(summary_table)
 
+    if missing_annotations or duplicate_uuids or duplicate_annotations:
+        console.print(
+            "[bold red]Dataset is unhealthy![/bold red] "
+            "Run [green]luxonis_ml data sanitize[/green] to automatically remove duplicates and missing entries."
+        )
+
     all_task_names = sorted(
         set(stats["class_distributions"].keys())
         | set(stats["heatmaps"].keys())
@@ -828,6 +856,28 @@ def clone(
         ),
     ] = True,
     bucket_storage: BucketStorage = bucket_option,
+    splits_to_clone: Annotated[
+        str | None,
+        typer.Option(
+            "--split",
+            "-s",
+            help=(
+                "Comma separated list of split names to clone, "
+                "e.g. `-s val,test` or just `-s train`. "
+                "If omitted, clones all splits."
+            ),
+            show_default=False,
+        ),
+    ] = None,
+    team_id: Annotated[
+        str | None,
+        typer.Option(
+            "--team-id",
+            "-t",
+            help="Team ID to use for the new dataset. If not provided, the dataset's current team ID will be used.",
+            show_default=False,
+        ),
+    ] = None,
 ):
     """Clone an existing dataset with a new name.
 
@@ -843,9 +893,21 @@ def clone(
     ):
         raise typer.Exit
 
+    if splits_to_clone:
+        split_list = [
+            s.strip() for s in splits_to_clone.split(",") if s.strip()
+        ]
+    else:
+        split_list = None
+
     print(f"Cloning dataset '{name}' to '{new_name}'...")
     dataset = LuxonisDataset(name, bucket_storage=bucket_storage)
-    dataset.clone(new_dataset_name=new_name, push_to_cloud=push_to_cloud)
+    dataset.clone(
+        new_dataset_name=new_name,
+        push_to_cloud=push_to_cloud,
+        splits_to_clone=split_list,
+        team_id=team_id,
+    )
     print(f"[green]Dataset '{name}' successfully cloned to '{new_name}'.")
 
 
@@ -877,7 +939,29 @@ def merge(
             show_default=False,
         ),
     ] = None,
+    splits_to_merge: Annotated[
+        str | None,
+        typer.Option(
+            "--split",
+            "-s",
+            help=(
+                "Comma separated list of split names to merge, "
+                "e.g. `-s val,test` or just `-s train`. "
+                "If omitted, merges all splits."
+            ),
+            show_default=False,
+        ),
+    ] = None,
     bucket_storage: BucketStorage = bucket_option,
+    team_id: Annotated[
+        str | None,
+        typer.Option(
+            "--team-id",
+            "-t",
+            help="Team ID to use for the new dataset. If not provided, the dataset's current team ID will be used.",
+            show_default=False,
+        ),
+    ] = None,
 ):
     """Merge two datasets stored in the same type of bucket."""
     check_exists(source_name, bucket_storage)
@@ -898,6 +982,13 @@ def merge(
     ):
         raise typer.Exit
 
+    if splits_to_merge:
+        split_list = [
+            s.strip() for s in splits_to_merge.split(",") if s.strip()
+        ]
+    else:
+        split_list = None
+
     source_dataset = LuxonisDataset(source_name, bucket_storage=bucket_storage)
     target_dataset = LuxonisDataset(target_name, bucket_storage=bucket_storage)
 
@@ -907,7 +998,11 @@ def merge(
     )
 
     _ = target_dataset.merge_with(
-        source_dataset, inplace=inplace, new_dataset_name=new_name
+        source_dataset,
+        inplace=inplace,
+        new_dataset_name=new_name,
+        splits_to_merge=split_list,
+        team_id=team_id,
     )
 
     if inplace:
@@ -920,5 +1015,18 @@ def merge(
         )
 
 
+@app.command()
+def sanitize(
+    name: DatasetNameArgument,
+    bucket_storage: BucketStorage = bucket_option,
+):
+    """Remove duplicate annotations and duplicate files from the
+    dataset."""
+    check_exists(name, bucket_storage)
+    dataset = LuxonisDataset(name, bucket_storage=bucket_storage)
+    dataset.remove_duplicates()
+    print(f"[green]Duplicates removed from dataset '{name}'.")
+
+
 if __name__ == "__main__":
     app()
