@@ -6,11 +6,12 @@ from typing import Any, Literal, TypeAlias
 
 import albumentations as A
 import numpy as np
+from albumentations.core.composition import TransformsSeqType
 from loguru import logger
 from typing_extensions import override
 
 from luxonis_ml.data.utils.task_utils import get_task_name, task_is_metadata
-from luxonis_ml.typing import ConfigItem, LoaderOutput, Params
+from luxonis_ml.typing import ConfigItem, LoaderMultiOutput, Params
 
 from .base_engine import AugmentationEngine
 from .batch_compose import BatchCompose
@@ -27,6 +28,7 @@ from .utils import (
 
 Data: TypeAlias = dict[str, np.ndarray]
 TargetType: TypeAlias = Literal[
+    "image",
     "array",
     "classification",
     "mask",
@@ -277,16 +279,20 @@ class AlbumentationsEngine(AugmentationEngine, register_name="albumentations"):
         width: int,
         targets: dict[str, str],
         n_classes: dict[str, int],
+        source_names: list[str],
         config: Iterable[Params],
         keep_aspect_ratio: bool = True,
         is_validation_pipeline: bool = False,
         min_bbox_visibility: float = 0.0,
         seed: int | None = None,
+        bbox_area_threshold: float = 0.0004,
     ):
         self.targets: dict[str, TargetType] = {}
         self.target_names_to_tasks = {}
         self.n_classes = n_classes
         self.image_size = (height, width)
+        self.source_names = source_names
+        self.bbox_area_threshold = bbox_area_threshold
 
         for task, task_type in targets.items():
             target_name = self.task_to_target_name(task)
@@ -343,6 +349,9 @@ class AlbumentationsEngine(AugmentationEngine, register_name="albumentations"):
             self.targets[target_name] = target_type
             self.target_names_to_tasks[target_name] = task
 
+        for source_name in source_names:
+            self.targets[source_name] = "image"
+
         # Necessary for official Albumentations transforms.
         targets_without_instance_mask = {
             target_name: target_type
@@ -394,6 +403,20 @@ class AlbumentationsEngine(AugmentationEngine, register_name="albumentations"):
                     f"Only subclasses of `A.BasicTransform` are allowed. "
                 )
 
+        wrapped_spatial_ops: TransformsSeqType = []
+        if "keypoints" in targets.values():
+            for op in spatial_transforms:
+                wrapped_spatial_ops.append(op)
+                wrapped_spatial_ops.append(
+                    A.Lambda(
+                        image=lambda img, **kw: img,
+                        keypoints=self._mark_invisible_keypoints,
+                        p=1.0,
+                    )
+                )
+        else:
+            wrapped_spatial_ops = spatial_transforms
+
         if resize_transform is None:
             if keep_aspect_ratio:
                 resize_transform = LetterboxResize(height=height, width=width)
@@ -422,10 +445,12 @@ class AlbumentationsEngine(AugmentationEngine, register_name="albumentations"):
                 batch_transforms, **get_params(is_custom=True)
             )
             self.spatial_transform = wrap_transform(
-                A.Compose(spatial_transforms, **get_params())
+                A.Compose(wrapped_spatial_ops, **get_params())
             )
             self.pixel_transform = wrap_transform(
-                A.Compose(pixel_transforms), is_pixel=True
+                A.Compose(pixel_transforms),
+                is_pixel=True,
+                source_names=source_names,
             )
             self.resize_transform = wrap_transform(
                 A.Compose([resize_transform], **get_params())
@@ -440,13 +465,14 @@ class AlbumentationsEngine(AugmentationEngine, register_name="albumentations"):
         return self.batch_transform.batch_size
 
     @override
-    def apply(self, input_batch: list[LoaderOutput]) -> LoaderOutput:
+    def apply(self, input_batch: list[LoaderMultiOutput]) -> LoaderMultiOutput:
         data_batch, n_keypoints = self.preprocess_batch(input_batch)
 
         data = self.batch_transform(data_batch)
 
         for target_name in list(data.keys()):
-            if data[target_name].size == 0:
+            value = data[target_name]
+            if isinstance(value, np.ndarray) and value.size == 0:
                 del data[target_name]
 
         data = self.spatial_transform(**data)
@@ -470,7 +496,7 @@ class AlbumentationsEngine(AugmentationEngine, register_name="albumentations"):
         return self.postprocess(data, n_keypoints)
 
     def preprocess_batch(
-        self, labels_batch: list[LoaderOutput]
+        self, labels_batch: list[LoaderMultiOutput]
     ) -> tuple[list[Data], dict[str, int]]:
         """Preprocess a batch of labels.
 
@@ -485,10 +511,24 @@ class AlbumentationsEngine(AugmentationEngine, register_name="albumentations"):
         bbox_counters = defaultdict(int)
         n_keypoints = {}
 
-        for image, labels in labels_batch:
-            data = {"image": image}
-            height, width, _ = image.shape
+        for image_dict, labels in labels_batch:
+            data = {}
+
+            key = next(iter(image_dict))
+            data["_original_image_key"] = key
+            for source_name, img in image_dict.items():
+                if source_name == key:
+                    data["image"] = img
+                else:
+                    data[source_name] = img
+
+            sample_img = next(iter(image_dict.values()))
+            height, width = sample_img.shape[:2]
+
             for target_name, target_type in self.targets.items():
+                if target_name not in self.target_names_to_tasks:
+                    continue
+
                 task = self.target_names_to_tasks[target_name]
 
                 if task not in labels:
@@ -537,7 +577,7 @@ class AlbumentationsEngine(AugmentationEngine, register_name="albumentations"):
 
     def postprocess(
         self, data: Data, n_keypoints: dict[str, int]
-    ) -> LoaderOutput:
+    ) -> LoaderMultiOutput:
         """Postprocess the augmented data back to LDF format.
 
         Discards labels associated with bboxes that are outside the
@@ -549,12 +589,26 @@ class AlbumentationsEngine(AugmentationEngine, register_name="albumentations"):
         @type n_keypoints: Dict[str, int]
         @param n_keypoints: Dictionary mapping task names to the number
             of keypoints for that task.
-        @rtype: LoaderOutput
-        @return: Tuple containing the augmented image and the labels.
+        @rtype: LoaderMultiOutput
+        @return: Tuple containing the augmented image dict and the
+            labels.
         """
         out_labels = {}
-        out_image = data.pop("image")
-        image_height, image_width, _ = out_image.shape
+        out_image_dict = {}
+
+        image_keys = [k for k in data if k in ["image", *self.source_names]]
+
+        for key in image_keys:
+            img = data.pop(key)
+            if img.ndim == 2:
+                img = np.expand_dims(img, axis=-1)
+            restored_key = (
+                data["_original_image_key"] if key == "image" else key
+            )
+            out_image_dict[restored_key] = img
+
+        sample_img = next(iter(out_image_dict.values()))
+        image_height, image_width = sample_img.shape[:2]
 
         bboxes_indices = {}
 
@@ -570,7 +624,9 @@ class AlbumentationsEngine(AugmentationEngine, register_name="albumentations"):
             task_name = get_task_name(task)
 
             if target_type == "bboxes":
-                out_labels[task], index = postprocess_bboxes(array)
+                out_labels[task], index = postprocess_bboxes(
+                    array, self.bbox_area_threshold
+                )
                 bboxes_indices[task_name] = index
 
         for target_name, target_type in self.targets.items():
@@ -617,7 +673,27 @@ class AlbumentationsEngine(AugmentationEngine, register_name="albumentations"):
             elif target_type == "classification":
                 out_labels[task] = array
 
-        return out_image, out_labels
+        return out_image_dict, out_labels
+
+    @staticmethod
+    def _mark_invisible_keypoints(
+        keypoints: np.ndarray, **kwargs
+    ) -> np.ndarray:
+        """
+        keypoints: np.ndarray of shape (N,6) columns = [x, y, z, a, s, v]
+        Zeroes out the visibility (last) column if (x,y) is out of image bounds.
+        """
+        shape = kwargs.get("shape")
+        if shape is None:
+            raise ValueError(
+                "Shape must be provided in kwargs to mark invisible keypoints."
+            )
+        h, w = shape[:2]
+        kps = keypoints.copy()
+        xs, ys = kps[:, 0], kps[:, 1]
+        oob = (xs < 0) | (ys < 0) | (xs >= w) | (ys >= h)
+        kps[oob, -1] = 0.0
+        return kps
 
     @staticmethod
     def create_transformation(
@@ -635,16 +711,41 @@ class AlbumentationsEngine(AugmentationEngine, register_name="albumentations"):
 
 
 def wrap_transform(
-    transform: A.BaseCompose, is_pixel: bool = False
+    transform: A.BaseCompose,
+    is_pixel: bool = False,
+    source_names: list[str] | None = None,
 ) -> Callable[..., Data]:
     def apply_transform(**data: np.ndarray) -> Data:
         if not transform.transforms:
             return data
 
         if is_pixel:
-            data["image"] = transform(image=data["image"])["image"]
+            if source_names is None:
+                raise ValueError(
+                    "source_names must be provided for pixel transformations."
+                )
+            replay_transform = A.ReplayCompose(transform.transforms)
+
+            result = replay_transform(image=data["image"])
+            data["image"] = result["image"]
+
+            replay = result["replay"]
+            for source_name in source_names:
+                if source_name == "image" or source_name not in data:
+                    continue
+                img = data[source_name]
+                if img.ndim == 3:
+                    data[source_name] = A.ReplayCompose.replay(
+                        replay, image=img
+                    )["image"]
+
             return data
 
-        return transform(**data)
+        original_key = data.pop("_original_image_key", None)
+        transformed = transform(**data)
+        if original_key is not None:
+            transformed["_original_image_key"] = original_key
+
+        return transformed
 
     return apply_transform
