@@ -25,6 +25,7 @@ from typing_extensions import Self, override
 from luxonis_ml.data.utils.parquet import ParquetRecord
 from luxonis_ml.typing import PathType, check_type
 from luxonis_ml.utils import BaseModelExtraForbid
+from luxonis_ml.utils.logging import log_once
 
 KeypointVisibility: TypeAlias = Literal[0, 1, 2]
 NormalizedFloat: TypeAlias = Annotated[float, Field(ge=0, le=1)]
@@ -535,6 +536,10 @@ class DatasetRecord(BaseModelExtraForbid):
             raise ValueError("DatasetRecord must have exactly one file")
         return next(iter(self.files.values()))
 
+    @property
+    def all_file_paths(self) -> list[FilePath]:
+        return list(self.files.values())
+
     @model_validator(mode="after")
     def validate_task_name_valid_identifier(self) -> Self:
         check_valid_identifier(self.task_name, label="Task name")
@@ -544,9 +549,9 @@ class DatasetRecord(BaseModelExtraForbid):
     @classmethod
     def validate_task_name(cls, values: dict[str, Any]) -> dict[str, Any]:
         if "task" in values:
-            warnings.warn(
+            log_once(
+                logger.warning,
                 "The 'task' field is deprecated. Use 'task_name' instead.",
-                stacklevel=2,
             )
             values["task_name"] = values.pop("task")
         return values
@@ -557,6 +562,11 @@ class DatasetRecord(BaseModelExtraForbid):
         values = deepcopy(values)
         if "file" in values:
             values["files"] = {"image": values.pop("file")}
+        if "files" in values:
+            files_dict = values["files"]
+            values["files"] = {
+                k: Path(v).resolve() for k, v in files_dict.items()
+            }
         return values
 
     def to_parquet_rows(self) -> Iterable[ParquetRecord]:
@@ -577,8 +587,11 @@ class DatasetRecord(BaseModelExtraForbid):
         @rtype: L{ParquetDict}
         @return: A dictionary of annotation data.
         """
-        for source, file_path in self.files.items():
-            if annotation is None:
+        file_items = sorted(self.files.items(), key=lambda x: str(x[1]))
+        for i, (source, file_path) in enumerate(file_items):
+            is_main = i == 0
+
+            if annotation is None or not is_main:
                 yield {
                     "file": str(file_path),
                     "source_name": source,
