@@ -10,6 +10,7 @@ from pytest_subtests.plugin import SubTests
 from luxonis_ml.data import (
     BucketStorage,
     Category,
+    LuxonisComponent,
     LuxonisDataset,
     LuxonisLoader,
     LuxonisParser,
@@ -103,9 +104,24 @@ def test_dataset(
         pytest.exit("Dataset creation failed")
 
     with subtests.test("test_source"):
-        assert dataset.source == LuxonisSource()
-        dataset.update_source(LuxonisSource(name="test"))
-        assert dataset.source == LuxonisSource(name="test")
+        assert dataset.source == LuxonisSource(
+            name="default",
+            components={"image": LuxonisComponent(name="image")},
+            main_component="image",
+        )
+        dataset.update_source(
+            LuxonisSource(
+                name="test",
+                components={"image": LuxonisComponent(name="image")},
+                main_component="image",
+            )
+        )
+        assert dataset.source == LuxonisSource(
+            name="test",
+            components={"image": LuxonisComponent(name="image")},
+            main_component="image",
+        )
+        assert dataset.get_source_names() == ["image"]
 
     with subtests.test("test_load"):
         loader = LuxonisLoader(dataset)
@@ -151,6 +167,9 @@ def test_dataset_fail(dataset_name: str, tempdir: Path):
     with pytest.raises(ValueError, match="Must provide either"):
         dataset.set_skeletons()
 
+    with pytest.raises(ValueError, match="Must set delete_remote"):
+        dataset.delete_dataset()
+
 
 @pytest.mark.dependency(name="test_dataset[BucketStorage.LOCAL]")
 def test_loader_iterator(storage_url: str, tempdir: Path):
@@ -237,6 +256,9 @@ def test_make_splits(
     with pytest.raises(ValueError, match="No new files"):
         dataset.make_splits()
 
+    with pytest.raises(ValueError, match="Splits cannot be empty"):
+        dataset.make_splits({})
+
     with pytest.raises(ValueError, match="Ratios must sum to 1.0"):
         dataset.make_splits((0.7, 0.1, 1))
 
@@ -249,11 +271,6 @@ def test_make_splits(
     with pytest.raises(ValueError, match="Ratios must sum to 1.0"):
         dataset.make_splits({"train": 1.5})
 
-    with pytest.raises(ValueError, match="Dataset size is smaller than"):
-        dataset.make_splits(
-            {split: defs * 2 for split, defs in splits.items()}
-        )
-
     dataset.add(generator(10))
     dataset.make_splits({"custom_split": 1.0})
     splits = dataset.get_splits()
@@ -764,6 +781,123 @@ def test_merge_datasets(
     )
 
 
+@pytest.mark.dependency(name="test_dataset[BucketStorage.LOCAL]")
+def test_merge_datasets_specific_split(
+    bucket_storage: BucketStorage,
+    dataset_name: str,
+    tempdir: Path,
+):
+    dataset_name = f"{dataset_name}_{bucket_storage.value}"
+    dataset1_name = f"{dataset_name}_1"
+    dataset2_name = f"{dataset_name}_2"
+
+    def generator1() -> DatasetIterator:
+        for i in range(3):
+            img = create_image(i, tempdir)
+            yield {
+                "file": img,
+                "annotation": {
+                    "class": "person",
+                    "boundingbox": {"x": 0.1, "y": 0.1, "w": 0.1, "h": 0.1},
+                },
+            }
+
+    def generator2() -> DatasetIterator:
+        for i in range(3, 6):
+            img = create_image(i, tempdir)
+            yield {
+                "file": img,
+                "annotation": {
+                    "class": "dog",
+                    "boundingbox": {"x": 0.2, "y": 0.2, "w": 0.2, "h": 0.2},
+                },
+            }
+
+    dataset1 = create_dataset(
+        dataset1_name,
+        generator1(),
+        bucket_storage,
+        splits={"train": 0.6, "val": 0.4},
+    )
+
+    dataset2 = create_dataset(
+        dataset2_name,
+        generator2(),
+        bucket_storage,
+        splits={"train": 0.6, "val": 0.4},
+    )
+
+    merged_dataset = dataset1.merge_with(
+        dataset2,
+        inplace=False,
+        new_dataset_name=f"{dataset1_name}_{dataset2_name}_merged",
+        splits_to_merge=["train"],
+    )
+
+    merged_stats = merged_dataset.get_statistics()
+    assert {
+        (item["count"], item["class_name"])
+        for item in merged_stats["class_distributions"][""]["boundingbox"]
+    } == {(3, "person"), (2, "dog")}
+    merged_splits = merged_dataset.get_splits()
+    dataset1_splits = dataset1.get_splits()
+    dataset2_splits = dataset2.get_splits()
+    assert merged_splits is not None
+    assert dataset1_splits is not None
+    assert dataset2_splits is not None
+    assert set(merged_splits["train"]) == set(dataset1_splits["train"]) | set(
+        dataset2_splits["train"]
+    )
+    assert set(merged_splits["val"]) == set(dataset1_splits["val"])
+
+    dataset1.delete_dataset(delete_local=True, delete_remote=True)
+    dataset2.delete_dataset(delete_local=True, delete_remote=True)
+    merged_dataset.delete_dataset(delete_local=True, delete_remote=True)
+
+
+@pytest.mark.dependency(name="test_dataset[BucketStorage.LOCAL]")
+def test_clone_dataset_specific_split(
+    bucket_storage: BucketStorage,
+    dataset_name: str,
+    tempdir: Path,
+):
+    def generator() -> DatasetIterator:
+        for i in range(3):
+            img = create_image(i, tempdir)
+            yield {
+                "file": img,
+                "annotation": {
+                    "class": "person",
+                    "boundingbox": {"x": 0.1, "y": 0.1, "w": 0.1, "h": 0.1},
+                },
+            }
+
+    dataset = create_dataset(
+        dataset_name,
+        generator(),
+        bucket_storage,
+        splits={"train": 0.6, "val": 0.4},
+    )
+    cloned_dataset = dataset.clone(
+        new_dataset_name=f"{dataset_name}_cloned",
+        splits_to_clone=["train"],
+    )
+    dataset_splits = dataset.get_splits()
+    cloned_splits = cloned_dataset.get_splits()
+    assert cloned_splits is not None
+    assert dataset_splits is not None
+    assert set(cloned_splits["train"]) == set(dataset_splits["train"])
+    assert "val" not in cloned_splits
+
+    cloned_stats = cloned_dataset.get_statistics()
+    assert {
+        (item["count"], item["class_name"])
+        for item in cloned_stats["class_distributions"][""]["boundingbox"]
+    } == {(2, "person")}
+
+    cloned_dataset.delete_dataset(delete_local=True, delete_remote=True)
+
+
 @pytest.mark.dependency(name="test_dataset[BucketStorage.LOCAL]")
 def test_classes_per_task(dataset_name: str, tempdir: Path):
     def generator() -> DatasetIterator:
@@ -926,3 +1060,215 @@ def test_dataset_push_pull(
         loader = LuxonisLoader(cloud_dataset)
         assert sum(1 for _ in loader) == 3
         assert cloud_dataset.get_statistics() == original_stats
+
+
+@pytest.mark.dependency(name="test_dataset[BucketStorage.LOCAL]")
+def test_merge_on_different_machines(dataset_name: str, tempdir: Path):
+    def generator(start: int, end: int) -> DatasetIterator:
+        """Generate sample dataset items with bounding boxes."""
+        for i in range(start, end):
+            img = create_image(i, tempdir)
+            yield {
+                "file": img,
+                "annotation": {
+                    "class": "person",
+                    "boundingbox": {"x": 0.1, "y": 0.1, "w": 0.1, "h": 0.1},
+                    "instance_id": i,
+                },
+            }
+
+    dataset1 = create_dataset(
+        dataset_name + "_1",
+        generator(0, 3),
+        bucket_storage=BucketStorage.GCS,
+        delete_local=True,
+        delete_remote=True,
+        splits=(1, 0, 0),
+    )
+    dataset2 = create_dataset(
+        dataset_name + "_2",
+        generator(3, 6),
+        bucket_storage=BucketStorage.GCS,
+        delete_local=True,
+        delete_remote=True,
+        splits=(1, 0, 0),
+    )
+    shutil.rmtree(tempdir)
+    dataset1.pull_from_cloud()
+    dataset2.pull_from_cloud()
+    dataset1.delete_dataset(delete_remote=True)
+    dataset2.delete_dataset(delete_remote=True)
+    dataset1 = LuxonisDataset(dataset_name + "_1")
+    dataset2 = LuxonisDataset(dataset_name + "_2")
+    assert len(list(dataset1.media_path.glob("*"))) == 3
+    assert len(list(dataset2.media_path.glob("*"))) == 3
+    dataset3 = dataset1.merge_with(
+        dataset2, inplace=False, new_dataset_name=dataset_name
+    )
+    loader = LuxonisLoader(dataset3)
+    assert sum(1 for _ in loader) == 6
+    dataset3.export(tempdir)
+    assert (
+        len(
+            list(
+                Path.cwd().glob(
+                    f"{tempdir}/{dataset3.dataset_name}/train/images/*"
+                )
+            )
+        )
+        == 6
+    )
+
+
+def create_test_dataset_with_classes(
+    tempdir: Path, task_classes: dict[str, dict[str, int]]
+) -> LuxonisDataset:
+    """Helper function to create a test dataset with specific class
+    mappings."""
+
+    def generator() -> DatasetIterator:
+        for i in range(5):
+            img = create_image(i, tempdir)
+            yield {
+                "file": img,
+                "annotation": {
+                    "class": list(task_classes["classification"].keys())[
+                        i % len(task_classes["classification"])
+                    ],
+                    "boundingbox": {
+                        "x": 0.1 + i * 0.1,
+                        "y": 0.1 + i * 0.1,
+                        "w": 0.2,
+                        "h": 0.2,
+                    },
+                },
+                "task_name": "classification",
+            }
+
+    dataset = LuxonisDataset(
+        "test_class_order",
+        delete_local=True,
+        delete_remote=True,
+        bucket_storage=BucketStorage.LOCAL,
+    ).add(generator())
+
+    # Set the classes for the dataset
+    for task_name, classes in task_classes.items():
+        dataset.set_classes(classes, task=task_name)
+
+    dataset.make_splits(ratios=(1, 0, 0))
+    return dataset
+
+
+def test_class_order_per_task_valid_reordering(tempdir: Path):
+    """Test valid class reordering for a task."""
+    original_classes = {"classification": {"cat": 0, "dog": 1, "bird": 2}}
+
+    dataset = create_test_dataset_with_classes(tempdir, original_classes)
+
+    # Define new class order
+    class_order_per_task = {"classification": ["dog", "bird", "cat"]}
+    dataset.set_class_order_per_task(class_order_per_task)
+
+    # Verify that classes were reordered
+    expected_classes = {"dog": 0, "bird": 1, "cat": 2}
+    assert dataset.get_classes()["classification"] == expected_classes
+
+
+def test_class_order_per_task_multiple_tasks(tempdir: Path):
+    """Test class reordering for multiple tasks."""
+    original_classes = {
+        "classification": {"cat": 0, "dog": 1, "bird": 2},
+        "detection": {"person": 0, "car": 1, "bike": 2},
+    }
+
+    # Create a more complex dataset with multiple tasks
+    def generator() -> DatasetIterator:
+        for i in range(5):
+            img = create_image(i, tempdir)
+            yield {
+                "file": img,
+                "annotation": {
+                    "class": list(original_classes["classification"].keys())[
+                        i % 3
+                    ],
+                },
+                "task_name": "classification",
+            }
+
+        for i in range(5, 10):
+            img = create_image(i, tempdir)
+            yield {
+                "file": img,
+                "annotation": {
+                    "class": list(original_classes["detection"].keys())[
+                        (i - 5) % 3
+                    ],
+                },
+                "task_name": "classification_1",
+            }
+
+    dataset = LuxonisDataset(
+        "test_multi_task_class_order",
+        delete_local=True,
+        delete_remote=True,
+        bucket_storage=BucketStorage.LOCAL,
+    ).add(generator())
+
+    for task_name, classes in original_classes.items():
+        dataset.set_classes(classes, task=task_name)
+
+    dataset.make_splits(ratios=(1, 0, 0))
+
+    # Define new class orders for both tasks
+    class_order_per_task = {
+        "classification": ["bird", "cat", "dog"],
+        "classification_1": ["bike", "person", "car"],
+    }
+
+    dataset.set_class_order_per_task(class_order_per_task)
+
+    # Verify both tasks were reordered correctly
+    expected_classification = {"bird": 0, "cat": 1, "dog": 2}
+    expected_detection = {"bike": 0, "person": 1, "car": 2}
+
+    assert dataset.get_classes()["classification"] == expected_classification
+    assert dataset.get_classes()["classification_1"] == expected_detection
+
+
+def test_class_order_per_task_invalid_task_name(tempdir: Path):
+    """Test error when providing an invalid task name."""
+    original_classes = {"classification": {"cat": 0, "dog": 1, "bird": 2}}
+
+    dataset = create_test_dataset_with_classes(tempdir, original_classes)
+
+    # Define class order for non-existent task
+    class_order_per_task = {"invalid_task": ["cat", "dog", "bird"]}
+
+    with pytest.raises(
+        ValueError,
+        match=r"Task invalid_task not found in dataset tasks\. Available tasks: \['classification'\]",
+    ):
+        dataset.set_class_order_per_task(class_order_per_task)
+
+
+def test_class_order_per_task_mismatched_classes(tempdir: Path):
+    """Test error when provided classes don't match dataset classes."""
+    original_classes = {"classification": {"cat": 0, "dog": 1, "bird": 2}}
+
+    dataset = create_test_dataset_with_classes(tempdir, original_classes)
+
+    # Define class order with wrong class names
+    class_order_per_task = {
+        "classification": [
+            "cat",
+            "dog",
+            "fish",
+        ]  # "fish" is not in original classes
+    }
+
+    with pytest.raises(
+        ValueError,
+        match=r"Classes for task classification do not match the classes in the dataset.",
+    ):
+        dataset.set_class_order_per_task(class_order_per_task)
